"""
stories.contrib.sentry
----------------------

This module contains integration with Sentry.

:copyright: (c) 2018-2020 dry-python team.
:license: BSD, see LICENSE for more details.
"""
import _stories.contrib.sentry  # noqa


__all__ = []
