# type: ignore
"""
stories.contrib.debug_toolbars.django
-------------------------------------

This module contains integration with Django Debug Toolbar.

:copyright: (c) 2018-2020 dry-python team.
:license: BSD, see LICENSE for more details.
"""
from _stories.contrib.debug_toolbars.django.panels import StoriesPanel


__all__ = ["StoriesPanel"]
